
yarip.Overlay = function()
{
	const PROFILE = "extensions/yarip@mozdev.org";
	const XPATH = PROFILE + "/xpath.xml";
	const CHARSET = "UTF-8";

	this.enabled = true;
	this.active = false;

	this.loader = null;
	this.handler = null;
	this.pages = null;
	this.mode = null;
	this.deep = null;

	this.start = function(event)
	{
		if (event.button != 0) return;

		var wasActive = this.active;
		this.stop(wasActive ? false : true);
		if (wasActive) return;

		this.toggleEnabled(true);
		var doc = gBrowser.selectedBrowser.contentDocument;
		if (!doc) return;

//		this.handler = _content.handler;
		this.handler = doc.defaultView.handler;
		if (!this.handler) return;

		var handlerWorks = this.handler.start(doc, this.pages, this.mode);
		if (!handlerWorks)
		{
			this.stop();
			return;
		}

		this.active = true;
		document.getElementById("yarip-statusbarpanel").setAttribute("status", "started");
	};

	this.stop = function(noStateChange)
	{
		if (!this.active || !this.handler) return;

		this.handler.stop();
		this.active = false;

		if (noStateChange) return;
		document.getElementById("yarip-statusbarpanel").setAttribute("status", "stopped");
	};

	this.save = function()
	{
		this.stop(true);

		var file = this.getFile(XPATH);
		if (!file) return;
		this.savePages(file);

		document.getElementById("yarip-statusbarpanel").setAttribute("status", "saved");
	};

	this.reset = function(mode, noStateChange)
	{
		this.stop(true);
		this.toggleEnabled(true);

		var node = document.popupNode;
		if (!node) return;
		var doc = node.ownerDocument;
		if (!doc) return;
		if (typeof mode != "number")
		{
			var aAddress = yarip.getAddressArray(doc, this.pages);
			for (var i = aAddress.length - 1; i >= 0; i--) this.pages[aAddress[i]] = null;
			if (noStateChange) return;
			document.getElementById("yarip-statusbarpanel").setAttribute("status", "reset");
			return;
		}

		var address = yarip.getAddress(doc, mode);
		if (!address || !this.pages || !this.pages[address]) return;
//		this.pages[address].reset();
		this.pages[address] = null;

		if (noStateChange) return;
		document.getElementById("yarip-statusbarpanel").setAttribute("status", "reset");
	};

	this.reload = function()
	{
		this.stop(true);
		this.toggleEnabled(true);

		var file = this.getFile(XPATH);
		if (!file) return;
		this.pages = this.loadPages(file);

		document.getElementById("yarip-statusbarpanel").setAttribute("status", "reloaded");
	};

	this.whitelist = function()
	{
		this.stop(true);

		var node = document.popupNode;
		if (!node) return;

		var doc = node.ownerDocument;
		var address = yarip.getAddress(doc, this.mode);
		if (!doc || !address) return;

		var xValue = yarip.calculateXPath(node);
		if (!xValue) return;
		xValue = prompt("Edit XPath for whitelisting", xValue);
		if (!xValue) return;

		var elements = yarip.getElementsByXPath(doc, xValue);
		if (!elements) return;
		for (var i = 0; i < elements.snapshotLength; i++)
		{
			var element = elements.snapshotItem(i);
			while (element && element.nodeType == 1)
			{
				element.setAttribute("status", "whitelisted");
				element = element.parentNode;
			}
		}

		if (!this.pages[address]) this.pages[address] = new yarip.Page(address);
		this.pages[address].whitelist.addXPath(new yarip.XPath(xValue, this.deep));

		document.getElementById("yarip-statusbarpanel").setAttribute("status", "whitelisted");
	};

	this.blacklist = function()
	{
		this.stop(true);

		var node = document.popupNode;
		if (!node) return;

		var doc = node.ownerDocument;
		var address = yarip.getAddress(doc, this.mode);
		if (!address) return;

		var xValue = yarip.calculateXPath(node);
		if (!xValue) return;
		xValue = prompt("Edit XPath for blacklisting", xValue);
		if (!xValue) return;

		yarip.removeByXPath(doc, this.pages, address, new yarip.XPath(xValue, this.deep), true);

		document.getElementById("yarip-statusbarpanel").setAttribute("status", "blacklisted");
	};

	this.setMode = function(mode, clear)
	{
		if (typeof mode != "number") return;
		this.stop(true);

		if (clear)
		{
			document.getElementById("yarip-page-menuitem").setAttribute("checked", "false");
			document.getElementById("yarip-fqdn-menuitem").setAttribute("checked", "false");
			document.getElementById("yarip-sld-menuitem").setAttribute("checked", "false");
			document.getElementById("yarip-user-menuitem").setAttribute("checked", "false");
		}

		switch (mode)
		{
			case yarip.MODES.page:
				document.getElementById("yarip-page-menuitem").setAttribute("checked", "true");
				break;

			case yarip.MODES.fqdn:
				document.getElementById("yarip-fqdn-menuitem").setAttribute("checked", "true");
				break;

			case yarip.MODES.sld:
				document.getElementById("yarip-sld-menuitem").setAttribute("checked", "true");
				break;

			case yarip.MODES.user:
				var doc = gBrowser.selectedBrowser.contentDocument;
				var address = yarip.getAddress(doc, this.mode);
				if (!address)
				{
					yarip.address = null;
					if (this.mode != yarip.MODES.user) this.setMode(this.mode, true);
					return;
				}
				address = prompt("Edit address", address);
				if (!address)
				{
					yarip.address = null;
					if (this.mode != yarip.MODES.user) this.setMode(this.mode, true);
					return;
				}
				yarip.address = address;
				document.getElementById("yarip-user-menuitem").setAttribute("checked", "true");
		}

		this.mode = mode;
	};

	this.toggleEnabled = function(value)
	{
		if (value == true || value == false) this.enabled = value;
		else this.enabled = !this.enabled;

		if (!this.enabled) this.stop(true);

		document.getElementById("yarip-enabled-menuitem").setAttribute("checked", "" + this.enabled);
		document.getElementById("yarip-statusbarpanel").setAttribute("status", this.enabled ? "enabled" : "disabled");
	};

	this.toggleDeep = function(value)
	{
		if (value == true || value == false) this.deep = value;
		else this.deep = !this.deep;

		document.getElementById("yarip-deep-menuitem").setAttribute("checked", "" + this.deep);
	};

	this.handleEvent = function(event)
	{
		switch(event.type)
		{
			case "load":
				this.stop();
				this.pages = this.loadPages();
				this.toggleEnabled(true);
				this.toggleDeep(true);
				this.setMode(yarip.MODES.fqdn, true);
				this.loader = new yarip.Loader(this);
				document.getElementById("appcontent").addEventListener("DOMContentLoaded", this, false);
				break;

			case "unload":
				this.stop();
				this.save();
				break;

			case "DOMContentLoaded":
				this.stop();
				var doc = event.originalTarget;

				// loader
				if (doc && doc.nodeName == "#document" && this.enabled && this.pages)
				{
					this.loader.load(doc, this.pages);
					if (this.mode == yarip.MODES.user) this.setMode(yarip.MODES.fqdn, true);
				}

				// handler
				this.toggleDeep(true);
				if (!doc.defaultView.frameElement) doc.defaultView.handler = new yarip.Handler(this);
				else
				{
					var rootDoc = doc;
					while (rootDoc.defaultView.frameElement) rootDoc = rootDoc.defaultView.frameElement.ownerDocument; // get root doc
					if (!rootDoc.defaultView.handler) break;
					var handler = new yarip.Handler(this, doc);
					rootDoc.defaultView.handler.addHandler(handler);
				}
		}
	};

	this.loadPages = function(file)
	{
		if (!file) file = this.getFile(XPATH);

		var doc = this.getDoc(file);
		if (!doc || doc.documentElement.nodeName == "parsererror") return null;

		var pages = {};
		var iPage = doc.evaluate("./yarip/page", doc, null, XPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
		var nPage = iPage.iterateNext();
		while (nPage)
		{
			var pName = nPage.getAttribute("name");
			if (!pages[pName]) pages[pName] = new yarip.Page(pName);

			// whitelist
			var iWhitelist = doc.evaluate("./whitelist", nPage, null, XPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
			var nWhitelist = iWhitelist.iterateNext();
			if (nWhitelist)
			{
				var iXPath = doc.evaluate("./xpath", nWhitelist, null, XPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
				var nXPath = iXPath.iterateNext();
				while (nXPath)
				{
					pages[pName].whitelist.addXPath(new yarip.XPath(
						nXPath.getAttribute("value"),
						nXPath.getAttribute("deep"),
						nXPath.getAttribute("found"),
						nXPath.getAttribute("notFound"),
						nXPath.getAttribute("notRemoved")));
					nXPath = iXPath.iterateNext();
				}
			}

			// blacklist
			var iBlacklist = doc.evaluate("./blacklist", nPage, null, XPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
			var nBlacklist = iBlacklist.iterateNext();
			if (nBlacklist)
			{
				var iXPath = doc.evaluate("./xpath", nBlacklist, null, XPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
				var nXPath = iXPath.iterateNext();
				while (nXPath)
				{
					pages[pName].blacklist.addXPath(new yarip.XPath(
						nXPath.getAttribute("value"),
						nXPath.getAttribute("deep"),
						nXPath.getAttribute("found"),
						nXPath.getAttribute("notFound"),
						nXPath.getAttribute("notRemoved")));
					nXPath = iXPath.iterateNext();
				}
			}

			nPage = iPage.iterateNext();
		}

		return pages;
	};

	this.savePages = function(file)
	{
		if (!file) return;

		var aPage = [];
		for (var pName in this.pages)
		{
			if (!this.pages[pName]) continue;
			this.pages[pName].whitelist.sortXPaths();
			this.pages[pName].blacklist.sortXPaths();
			aPage.push(this.pages[pName]);
		}
		aPage.sort(yarip.PageSorter);

		this.pages = {};
		for (var i = 0; i < aPage.length; i++) this.pages[aPage[i].name] = aPage[i];

		var data = this.generateXml();
		this.saveToFile(data, file);
	};

	this.generateXml = function()
	{
		var r = "<?xml version=\"1.0\" encoding=\"" + CHARSET + "\"?>\n<yarip>\n";
		for (var pName in this.pages) r += this.pages[pName].generateXml();
		r += "</yarip>";
		return r;
	};

	this.getDoc = function(file)
	{
		if (!file) return null;
		var data = this.getData(file);
		var parser = new DOMParser();
		return parser.parseFromString(data, "text/xml");
	};

	this.getData = function(file)
	{
		var fiStream = Components.classes['@mozilla.org/network/file-input-stream;1'].createInstance(Components.interfaces.nsIFileInputStream);
		fiStream.init(file, 0x01, 00004, null);
		var siStream = Components.classes['@mozilla.org/scriptableinputstream;1'].createInstance(Components.interfaces.nsIScriptableInputStream);
		siStream.init(fiStream);
		var data = siStream.read(siStream.available());
		var suc = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"].createInstance(Components.interfaces.nsIScriptableUnicodeConverter);
		suc.charset = CHARSET;
		var data = suc.ConvertToUnicode(data);
		siStream.close();
		fiStream.close();
		return data;
	};

	this.getFile = function(path)
	{
		var file = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties).get("ProfD", Components.interfaces.nsIFile);
		var aPath = path.split(/\/|\\/);
		for (var i = 0; i < aPath.length; i++) file.append(aPath[i]);
		if (!file.exists())
		{
			file.create(Components.interfaces.nsIFile.NORMAL_FILE_TYPE, 0600);
			this.saveToFile("<?xml version=\"1.0\" encoding=\"" + CHARSET + "\"?>\n<yarip>\n</yarip>", file);
		}
		if (file.isFile()) return file;
		else return null;
	};

	this.saveToFile = function(data, file)
	{
		if (!data || !file) return;
		var fos = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);
		fos.init(file, 0x02 | 0x08 | 0x20, 0600, 0);
		var parser = new DOMParser();
		var doc = parser.parseFromString(data, "text/xml");
		var serializer = new XMLSerializer();
		serializer.serializeToStream(doc, fos, CHARSET);
		fos.close();
	};
};
